@@ -139,11 +139,22 @@ struct intel_rdt_s
   bool enabled;
 };
 
+struct net_devices_s
+{
+  bool enabled;
+};
+
 struct mount_ext_info_s
 {
   struct idmap_info_s idmap;
 };
 
+struct memory_policy_info_s
+{
+  char **mode;
+  char **flags;
+};
+
 struct linux_info_s
 {
   char **namespaces;
@@ -154,6 +165,8 @@ struct linux_info_s
   struct selinux_info_s selinux;
   struct mount_ext_info_s mount_ext;
   struct intel_rdt_s intel_rdt;
+  struct net_devices_s net_devices;
+  struct memory_policy_info_s memory_policy;
 };
 
 struct annotations_info_s
@@ -182,6 +195,7 @@ struct libcrun_checkpoint_restore_s
   char *work_path;
   bool leave_running;
   bool tcp_established;
+  bool tcp_close;
   bool shell_job;
   bool ext_unix_sk;
   bool detach;
@@ -271,6 +285,7 @@ struct libcrun_intel_rdt_update
 {
   const char *l3_cache_schema;
   const char *mem_bw_schema;
+  char *const *schemata;
 };
 
 LIBCRUN_PUBLIC int libcrun_container_update_intel_rdt (libcrun_context_t *context, const char *id,
@@ -295,6 +310,12 @@ LIBCRUN_PUBLIC int libcrun_container_read_pids (libcrun_context_t *context, cons
 
 LIBCRUN_PUBLIC int libcrun_write_json_containers_list (libcrun_context_t *context, FILE *out, libcrun_error_t *err);
 
+LIBCRUN_PUBLIC int libcrun_container_add_mounts_from_file (libcrun_context_t *context, const char *id, const char *file,
+                                                           libcrun_error_t *err);
+
+LIBCRUN_PUBLIC int libcrun_container_remove_mounts_from_file (libcrun_context_t *context, const char *id, const char *file,
+                                                              libcrun_error_t *err);
+
 // Not part of the public API, just a method in container.c we need to access from linux.c
 void get_root_in_the_userns (runtime_spec_schema_config_schema *def, uid_t host_uid, gid_t host_gid,
                              uid_t *uid, gid_t *gid);
