@@ -443,7 +443,12 @@ calculate_seccomp_checksum (runtime_spec_schema_config_linux_seccomp *seccomp, u
   blake3_hasher_finalize (&hasher, hash, sizeof (hash));
 
   for (i = 0; i < 32; i++)
-    sprintf (&out[i * 2], "%02x", hash[i]);
+    {
+      size_t remaining = sizeof (seccomp_checksum_t) - i * 2;
+      ret = snprintf (&out[i * 2], remaining, "%02x", hash[i]);
+      if (UNLIKELY (ret != 2))
+        return crun_make_error (err, 0, "internal error: static buffer has wrong size");
+    }
   out[64] = 0;
 
 #undef PROCESS_STRING
@@ -584,7 +589,7 @@ store_seccomp_cache (struct libcrun_seccomp_gen_ctx_s *ctx, libcrun_error_t *err
   if (is_empty_string (ctx->checksum))
     return 0;
 
-  dirfd = open_rundir_dirfd (container->context->state_root, err);
+  dirfd = open_rundir_dirfd ((container->context ? container->context->state_root : NULL), err);
   if (UNLIKELY (dirfd < 0))
     return dirfd;
 
@@ -869,7 +874,7 @@ libcrun_open_seccomp_bpf (struct libcrun_seccomp_gen_ctx_s *ctx, int *fd, libcru
   if (container == NULL || container->context == NULL)
     return crun_make_error (err, EINVAL, "invalid internal state");
 
-  dirfd = open_rundir_dirfd (container->context->state_root, err);
+  dirfd = open_rundir_dirfd ((container->context ? container->context->state_root : NULL), err);
   if (UNLIKELY (dirfd < 0))
     return dirfd;
 
